{
WMKeys.inc
Various type and constant declarations relating to keys that Winmacro recognizes.

Written by Ken Henderson.

Copyright (c) 1994 Ken Henderson.  All Rights Reserved.

Last modification: 5-20-95

}

type

  {Establishes the available shift and toggle keys}

  ShiftKey = (SKCtrl, SKAlt, SKShift, SKCapslock, SKNumlock, SKScrollLock);

  {Possible combinations of shift keys -- note that some combinations overlap,
  i.e., Shift-Alt and Alt-Shift are identical, so there are fewer than you might
  think.  This enumerated type is used to build a comprehensive list of available
  keys in both the component and in its property editor.}

  PossibleShifts = (PSNone, PSCtrl, PSAlt, PSShift, PSCtrlShift, PSCtrlAlt, PSShiftAlt, PSCtrlShiftAlt);

  ToggleKeySet = set of ShiftKey;

const

  {Bit flags used by Winmacro for each of the shift and toggle keys}

  CtrlOn =        $01;
  AltOn =         $02;
  ShiftOn =       $04;
  CapslockOn =    $08;
  NumlockOn =     $10;
  ScrollLockOn =  $20;

  {The following (stored as part of Toggles in TMHDR) prevents the ScrapMacro
  from being accidentally deallocated or used in other inappropriate situations.
  It accomplishes this without the need for yet another flag variable}

  ScrapMacroStatusBit = $80;

  {This array sets up the flag representative of each shift key}

  ShiftStateOn : array[ShiftKey] of Byte = (CtrlOn, AltOn, ShiftOn, CapslockOn, NumlockOn, ScrollLockOn);

  {This array is defined using the PossibleShifts enum from above.  It consists
  of bit flags OR'ed together to create shift status bytes corresponding to the
  possible shift states established by the enum.  This array is later used to
  set the high order byte of a temporary MKey from within a loop in order to
  build a list of all possible keys.}

  PossibleShiftStates : array[PossibleShifts] of Byte =
  (0, CtrlOn, AltOn, ShiftOn, CtrlOn or ShiftOn, CtrlOn or AltOn, ShiftOn or AltOn,
   CtrlOn or AltOn or ShiftOn);

   {This set lists all possible unshifted keys.  I used a set here because the
   standard VK_XXXX constants don't define values for regular keys (e.g., alphabetic
   or numeric keys) since their virtual key code values are equal to their ASCII
   values.  Using a set allowed me to easily establish them.  Note the exclusion of
   the shift and toggle keys.  This list only lists primary keys, so shift or toggle
   keys would be inappropriate here.}

  VK_NULL=0;
  VK_SemiColon=186;
  VK_Equal=187;
  VK_Comma=188;
  VK_Minus=189;
  VK_Period=190;
  VK_Slash=191;
  VK_BackQuote=192;
  VK_LeftBracket=219;
  VK_BackSlash=220;
  VK_RightBracket=221;
  VK_Quote=222;
  VK_Last=VK_Quote;

  VKeys : Set of byte =
  [
  vk_LButton  ,
  vk_RButton  ,
  vk_Cancel   ,
  vk_MButton  ,
  vk_Back     ,
  vk_Tab      ,
  vk_Clear    ,
  vk_Return   ,
  {vk_Shift    ,      Shift/toggle keys aren't allowed for our use}
  {vk_Control  ,      Shift/toggle keys aren't allowed for our use}
  {vk_Menu     ,      Shift/toggle keys aren't allowed for our use}
  vk_Pause    ,
  {vk_Capital  ,      Shift/toggle keys aren't allowed for our use}
  vk_Escape   ,
  vk_Space    ,
  vk_Prior    ,
  vk_Next     ,
  vk_End      ,
  vk_Home     ,
  vk_Left     ,
  vk_Up       ,
  vk_Right    ,
  vk_Down     ,
  {vk_Select   ,      Not used for our purposes
  vk_Print    ,
  vk_Execute  ,
  vk_SnapShot ,}
  vk_Insert   ,
  vk_Delete   ,
  {vk_Help     ,        Not used for our purposes}
  ord('A')..ord('Z'),
  ord('0')..ord('9'),
  vk_NumPad0  ,
  vk_NumPad1  ,
  vk_NumPad2  ,
  vk_NumPad3  ,
  vk_NumPad4  ,
  vk_NumPad5  ,
  vk_NumPad6  ,
  vk_NumPad7  ,
  vk_NumPad8  ,
  vk_NumPad9  ,
  vk_Multiply ,
  vk_Add      ,
  {vk_Separator,         Not used for our purposes}
  vk_Subtract ,
  vk_Decimal  ,
  vk_Divide   ,
  vk_F1       ,
  vk_F2       ,
  vk_F3       ,
  vk_F4       ,
  vk_F5       ,
  vk_F6       ,
  vk_F7       ,
  vk_F8       ,
  vk_F9       ,
  vk_F10      ,
  vk_F11      ,
  vk_F12      ,
{  vk_F13      ,
  vk_F14      ,
  vk_F15      ,
  vk_F16      ,
  vk_F17      ,
  vk_F18      ,
  vk_F19      ,
  vk_F20      ,
  vk_F21      ,
  vk_F22      ,
  vk_F23      ,
  vk_F24           Not used for our purposes}
  {vk_NumLock  ,   Shift/toggle keys aren't allowed for our use}
  {vk_Scroll,      Shift/toggle keys aren't allowed for our use}
  VK_SemiColon,
  VK_Equal,
  VK_Comma,
  VK_Minus,
  VK_Period,
  VK_Slash,
  VK_BackQuote,
  VK_LeftBracket,
  VK_BackSlash,
  VK_RightBracket,
  VK_Quote
  ];

  {Used to determine whether a key is an extended key.  Windows' GetKeyNameText
   requires a bit be set to distinguish extended keys}

  ExtendedVKeys : set of byte =
  [VK_Up,
   VK_Down,
   VK_Left,
   VK_Right,
   VK_Home,
   VK_End,
   VK_Prior,  {PgUp}
   VK_Next,   {PgDn}
   VK_Insert,
   VK_Delete];

  {Used to determine whether a key pressed is a shift key}

  ShiftVKeys : set of byte =
  [VK_CONTROL,
   VK_MENU,
   VK_SHIFT];

type


  {Used by the MessageToKey array to translate mouse messages to VK's}

  TMouseButton = record
    VKey : Byte;
    ButtonUp : boolean;
  end;

const

  {The following array is used by the TranslateMouseMsg routine to turn
   mouse messages received by the mouse hook routine into Windows virtual
   key codes that can be used to locate corresponding MKeys.  It's interesting
   to note that we ignore button down messages, waiting instead for the
   button up.  This is consistent with the keyboard hook which likewise looks
   at key up messages, and then only when any appropriate shift key up messages
   have been seen}

  VK_NOKEY = 0;
  {Used to convert WM_NCMOUSEXXXX messages to their WM_MOUSEXXXX counterparts}
  MOUSEMSGDELTA = WM_MOUSEMOVE-WM_NCMOUSEMOVE;
  MessageToKey : array[WM_MOUSEFIRST..WM_MOUSELAST] of TMouseButton =
  ((VKey:VK_NOKEY; ButtonUp:false),
   (VKey:VK_LBUTTON; ButtonUp:false),
   (VKey:VK_LBUTTON; ButtonUp:true),
   (VKey:VK_LBUTTON; ButtonUp:true),{Left button down, up and double-clicked}
   (VKey:VK_RBUTTON; ButtonUp:false),
   (VKey:VK_RBUTTON; ButtonUp:true),
   (VKey:VK_RBUTTON; ButtonUp:true),{Right button down, up and double-clicked}
   (VKey:VK_MBUTTON; ButtonUp:false),
   (VKey:VK_MBUTTON; ButtonUp:true),
   (VKey:VK_MBUTTON; ButtonUp:true)); {Middle button down, up and double-clicked}

type
    TSendKey = record
      Name : String;
      VKey : Byte;
    end;

const

     {Array of keys that StringToMacro (and, hence, SendKeys) recognizes.
     If you add to this list, you must be sure to keep it sorted alphabetically
     by Name because a binary search routine is used to scan it.}

     MaxSendKeyRecs = 41;
     SendKeyRecs : array[1..MaxSendKeyRecs] of TSendKey =
     (
      (Name:'BKSP';            VKey:VK_BACK),
      (Name:'BS';              VKey:VK_BACK),
      (Name:'BACKSPACE';       VKey:VK_BACK),
      (Name:'BREAK';           VKey:VK_CANCEL),
      (Name:'CAPSLOCK';        VKey:VK_CAPITAL),
      (Name:'CLEAR';           VKey:VK_CLEAR),
      (Name:'DEL';             VKey:VK_DELETE),
      (Name:'DELETE';          VKey:VK_DELETE),
      (Name:'DOWN';            VKey:VK_DOWN),
      (Name:'END';             VKey:VK_END),
      (Name:'ENTER';           VKey:VK_RETURN),
      (Name:'ESC';             VKey:VK_ESCAPE),
      (Name:'ESCAPE';          VKey:VK_ESCAPE),
      (Name:'F1';              VKey:VK_F1),
      (Name:'F2';              VKey:VK_F2),
      (Name:'F3';              VKey:VK_F3),
      (Name:'F4';              VKey:VK_F4),
      (Name:'F5';              VKey:VK_F5),
      (Name:'F6';              VKey:VK_F6),
      (Name:'F7';              VKey:VK_F7),
      (Name:'F8';              VKey:VK_F8),
      (Name:'F9';              VKey:VK_F9),
      (Name:'F10';             VKey:VK_F10),
      (Name:'F11';             VKey:VK_F11),
      (Name:'F12';             VKey:VK_F12),
      (Name:'F13';             VKey:VK_F13),
      (Name:'F14';             VKey:VK_F14),
      (Name:'F15';             VKey:VK_F15),
      (Name:'F16';             VKey:VK_F16),
      (Name:'HELP';            VKey:VK_HELP),
      (Name:'HOME';            VKey:VK_HOME),
      (Name:'INS';             VKey:VK_INSERT),
      (Name:'LEFT';            VKey:VK_LEFT),
      (Name:'NUMLOCK';         VKey:VK_NUMLOCK),
      (Name:'PGDN';            VKey:VK_NEXT),
      (Name:'PGUP';            VKey:VK_PRIOR),
      (Name:'PRTSC';           VKey:VK_PRINT),
      (Name:'RIGHT';           VKey:VK_RIGHT),
      (Name:'SCROLLLOCK';      VKey:VK_SCROLL),
      (Name:'TAB';             VKey:VK_TAB),
      (Name:'UP';              VKey:VK_UP)
     );


